@@ -31,6 +31,7 @@
 #include "clockcheck.h"
 #include "foreign.h"
 #include "filter.h"
+#include "hash.h"
 #include "missing.h"
 #include "msg.h"
 #include "phc.h"
@@ -38,6 +39,8 @@
 #include "servo.h"
 #include "stats.h"
 #include "print.h"
+#include "rtnl.h"
+#include "sk.h"
 #include "tlv.h"
 #include "tsproc.h"
 #include "uds.h"
@@ -93,6 +96,7 @@ struct clock {
 	int pollfd_valid;
 	int nports; /* does not include the UDS port */
 	int last_port_number;
+	struct hash *index2port;
 	int free_running;
 	int freq_est_interval;
 	int grand_master_capable; /* for 802.1AS only */
@@ -268,8 +272,12 @@ void clock_destroy(struct clock *c)
 	LIST_FOREACH_SAFE(p, &c->ports, list, tmp) {
 		clock_remove_port(c, p);
 	}
+	if (c->pollfd[0].fd >= 0) {
+		rtnl_close(c->pollfd[0].fd);
+	}
 	port_close(c->uds_port);
 	free(c->pollfd);
+	hash_destroy(c->index2port, NULL);
 	if (c->clkid != CLOCK_REALTIME) {
 		phc_close(c->clkid);
 	}
@@ -314,7 +322,26 @@ static void clock_freq_est_reset(struct clock *c)
 	c->fest.origin1 = tmv_zero();
 	c->fest.ingress1 = tmv_zero();
 	c->fest.count = 0;
-};
+}
+
+static void clock_link_status(void *ctx, int index, int linkup)
+{
+	struct clock *c = ctx;
+	struct port *p;
+	char key[16];
+
+	snprintf(key, sizeof(key), "%d", index);
+	p = hash_lookup(c->index2port, key);
+	if (!p) {
+		return;
+	}
+	port_link_status_set(p, linkup);
+	if (linkup) {
+		port_dispatch(p, EV_FAULT_CLEARED, 0);
+	} else {
+		port_dispatch(p, EV_FAULT_DETECTED, 0);
+	}
+}
 
 static void clock_management_send_error(struct port *p,
 					struct ptp_message *msg, int error_id)
@@ -767,23 +794,44 @@ static int clock_add_port(struct clock *c, int phc_index,
 			  struct interface *iface)
 {
 	struct port *p, *piter, *lastp = NULL;
+	int fd, index;
+	char key[16];
 
-	if (clock_resize_pollfd(c, c->nports + 1))
+	if (clock_resize_pollfd(c, c->nports + 1)) {
 		return -1;
-	p = port_open(phc_index, timestamping, ++c->last_port_number,
-		      iface, c);
+	}
+	p = port_open(phc_index, timestamping, ++c->last_port_number, iface, c);
 	if (!p) {
 		/* No need to shrink pollfd */
 		return -1;
 	}
-	LIST_FOREACH(piter, &c->ports, list)
+	LIST_FOREACH(piter, &c->ports, list) {
 		lastp = piter;
-	if (lastp)
+	}
+	if (lastp) {
 		LIST_INSERT_AFTER(lastp, p, list);
-	else
+	} else {
 		LIST_INSERT_HEAD(&c->ports, p, list);
+	}
 	c->nports++;
 	clock_fda_changed(c);
+
+	/* Remember the index to port mapping, for link status tracking. */
+	fd = sk_interface_fd();
+	if (fd < 0) {
+		return -1;
+	}
+	index = sk_interface_index(fd, iface->name);
+	if (index < 0) {
+		return -1;
+	}
+	snprintf(key, sizeof(key), "%d", index);
+	if (hash_insert(c->index2port, key, p)) {
+		pr_err("failed to add port with index %d twice!", index);
+		return -1;
+	}
+	close(fd);
+
 	return 0;
 }
 
@@ -891,13 +939,14 @@ struct clock *clock_create(enum clock_type type, struct config *config,
 	}
 
 	if (!(c->dds.flags & DDS_TWO_STEP_FLAG)) {
-		switch (config_get_int(config, NULL, "time_stamping")) {
+		switch (timestamping) {
 		case TS_SOFTWARE:
 		case TS_LEGACY_HW:
 			pr_err("one step is only possible "
 			       "with hardware time stamping");
 			return NULL;
 		case TS_HARDWARE:
+			timestamping = TS_ONESTEP;
 			if (config_set_int(config, "time_stamping", TS_ONESTEP))
 				return NULL;
 			break;
@@ -907,7 +956,7 @@ struct clock *clock_create(enum clock_type type, struct config *config,
 	}
 
 	/* Check the time stamping mode on each interface. */
-	switch (config_get_int(config, NULL, "time_stamping")) {
+	switch (timestamping) {
 	case TS_SOFTWARE:
 		required_modes |= SOF_TIMESTAMPING_TX_SOFTWARE |
 			SOF_TIMESTAMPING_RX_SOFTWARE |
@@ -939,8 +988,7 @@ struct clock *clock_create(enum clock_type type, struct config *config,
 	/* determine PHC Clock index */
 	if (config_get_int(config, NULL, "free_running")) {
 		phc_index = -1;
-	} else if (config_get_int(config, NULL, "time_stamping") == TS_SOFTWARE ||
-		   config_get_int(config, NULL, "time_stamping") == TS_LEGACY_HW) {
+	} else if (timestamping == TS_SOFTWARE || timestamping == TS_LEGACY_HW) {
 		phc_index = -1;
 	} else if (phc_device) {
 		if (1 != sscanf(phc_device, "/dev/ptp%d", &phc_index)) {
@@ -1071,13 +1119,16 @@ struct clock *clock_create(enum clock_type type, struct config *config,
 	LIST_INIT(&c->ports);
 	c->last_port_number = 0;
 
-	/*
-	 * Create the UDS interface.
-	 */
 	if (clock_resize_pollfd(c, 0)) {
 		pr_err("failed to allocate pollfd");
 		return NULL;
 	}
+
+	/* Open a RT netlink socket. */
+	c->pollfd[0].fd = rtnl_open();
+	c->pollfd[0].events = POLLIN|POLLPRI;
+
+	/* Create the UDS interface. */
 	c->uds_port = port_open(phc_index, timestamping, 0, udsif, c);
 	if (!c->uds_port) {
 		pr_err("failed to open the UDS port");
@@ -1085,6 +1136,11 @@ struct clock *clock_create(enum clock_type type, struct config *config,
 	}
 	clock_fda_changed(c);
 
+	c->index2port = hash_create();
+	if (!c->index2port) {
+		pr_err("failed create index-to-port hash table");
+		return NULL;
+	}
 	/* Create the ports. */
 	STAILQ_FOREACH(iface, &config->interfaces, list) {
 		if (clock_add_port(c, phc_index, timestamping, iface)) {
@@ -1099,7 +1155,9 @@ struct clock *clock_create(enum clock_type type, struct config *config,
 		port_dispatch(p, EV_INITIALIZE, 0);
 	}
 	port_dispatch(c->uds_port, EV_INITIALIZE, 0);
-
+	if (c->pollfd[0].fd >= 0) {
+		rtnl_link_query(c->pollfd[0].fd);
+	}
 	return c;
 }
 
@@ -1164,9 +1222,12 @@ static int clock_resize_pollfd(struct clock *c, int new_nports)
 {
 	struct pollfd *new_pollfd;
 
-	/* Need to allocate one extra block of fds for uds */
+	/*
+	 * Need to allocate one descriptor for RT netlink and one
+	 * whole extra block of fds for UDS.
+	 */
 	new_pollfd = realloc(c->pollfd,
-			     (new_nports + 1) * N_CLOCK_PFD *
+			     (1 + (new_nports + 1) * N_CLOCK_PFD) *
 			     sizeof(struct pollfd));
 	if (!new_pollfd)
 		return -1;
@@ -1191,7 +1252,7 @@ static void clock_fill_pollfd(struct pollfd *dest, struct port *p)
 static void clock_check_pollfd(struct clock *c)
 {
 	struct port *p;
-	struct pollfd *dest = c->pollfd;
+	struct pollfd *dest = c->pollfd + 1;
 
 	if (c->pollfd_valid)
 		return;
@@ -1407,7 +1468,7 @@ int clock_poll(struct clock *c)
 	struct port *p;
 
 	clock_check_pollfd(c);
-	cnt = poll(c->pollfd, (c->nports + 1) * N_CLOCK_PFD, -1);
+	cnt = poll(c->pollfd, 1 + (c->nports + 1) * N_CLOCK_PFD, -1);
 	if (cnt < 0) {
 		if (EINTR == errno) {
 			return 0;
@@ -1419,7 +1480,13 @@ int clock_poll(struct clock *c)
 		return 0;
 	}
 
+	/* Check the RT netlink. */
 	cur = c->pollfd;
+	if (cur->revents & (POLLIN|POLLPRI)) {
+		rtnl_link_status(cur->fd, clock_link_status, c);
+	}
+
+	cur++;
 	LIST_FOREACH(p, &c->ports, list) {
 		/* Let the ports handle their events. */
 		for (i = err = 0; i < N_POLLFD && !err; i++) {
@@ -1438,10 +1505,15 @@ int clock_poll(struct clock *c)
 			}
 		}
 
-		/* Check the fault timer. */
+		/*
+		 * When the fault timer expires we clear the fault,
+		 * but only if the link is up.
+		 */
 		if (cur[N_POLLFD].revents & (POLLIN|POLLPRI)) {
 			clock_fault_timeout(p, 0);
-			port_dispatch(p, EV_FAULT_CLEARED, 0);
+			if (port_link_status_get(p)) {
+				port_dispatch(p, EV_FAULT_CLEARED, 0);
+			}
 		}
 
 		cur += N_CLOCK_PFD;
