@@ -34,6 +34,14 @@ struct ptp_message; /*forward declaration*/
 /** Opaque type. */
 struct clock;
 
+enum clock_type {
+	CLOCK_TYPE_ORDINARY   = 0x8000,
+	CLOCK_TYPE_BOUNDARY   = 0x4000,
+	CLOCK_TYPE_P2P        = 0x2000,
+	CLOCK_TYPE_E2E        = 0x1000,
+	CLOCK_TYPE_MANAGEMENT = 0x0800,
+};
+
 /**
  * Obtains a reference to the best foreign master of a clock.
  * @param c  The clock instance.
@@ -68,16 +76,14 @@ struct config *clock_config(struct clock *c);
  * Create a clock instance. There can only be one clock in any system,
  * so subsequent calls will destroy the previous clock instance.
  *
+ * @param type         Specifies which type of clock to create.
  * @param config       Pointer to the configuration database.
- * @param phc_index    PTP hardware clock device to use.
- *                     Pass -1 to select CLOCK_REALTIME.
- * @param ifaces       A queue of network interfaces.
- * @param dds          A pointer to a default data set for the clock.
+ * @param phc_device   PTP hardware clock device to use. Pass NULL for automatic
+ *                     selection based on the network interface.
  * @return             A pointer to the single global clock instance.
  */
-struct clock *clock_create(struct config *config, int phc_index,
-			   struct interfaces_head *ifaces,
-			   struct default_ds *dds);
+struct clock *clock_create(enum clock_type type, struct config *config,
+			   const char *phc_device);
 
 /**
  * Obtains a clock's default data set.
@@ -99,6 +105,13 @@ void clock_destroy(struct clock *c);
  */
 UInteger8 clock_domain_number(struct clock *c);
 
+/**
+ * Obtains a reference to the first port in the clock's list.
+ * @param c  The clock instance.
+ * @return   A pointer to a port, or NULL if no ports are present.
+ */
+struct port *clock_first_port(struct clock *c);
+
 /**
  * Provide the follow_up info TLV from a slave port.
  * @param c  The clock instance.
@@ -263,11 +276,11 @@ void clock_update_time_properties(struct clock *c, struct timePropertiesDS tds);
 struct clock_description *clock_description(struct clock *c);
 
 /**
- * Obtain the number of ports a clock has, excluding the UDS port.
+ * Obtain the type of a clock.
  * @param c  The clock instance.
- * @return   The number of ports.
+ * @return   One of the @ref clock_type enumeration values.
  */
-int clock_num_ports(struct clock *c);
+enum clock_type clock_type(struct clock *c);
 
 /**
  * Perform a sanity check on a time stamp made by a clock.
