@@ -180,6 +180,8 @@ struct config_item config_tab[] = {
 	PORT_ITEM_ENU("delay_filter", FILTER_MOVING_MEDIAN, delay_filter_enu),
 	PORT_ITEM_INT("delay_filter_length", 10, 1, INT_MAX),
 	PORT_ITEM_ENU("delay_mechanism", DM_E2E, delay_mech_enu),
+	GLOB_ITEM_INT("dscp_event", 0, 0, 63),
+	GLOB_ITEM_INT("dscp_general", 0, 0, 63),
 	GLOB_ITEM_INT("domainNumber", 0, 0, 127),
 	PORT_ITEM_INT("egressLatency", 0, INT_MIN, INT_MAX),
 	PORT_ITEM_INT("fault_badpeernet_interval", 16, INT32_MIN, INT32_MAX),
@@ -534,7 +536,6 @@ int config_read(char *name, struct config *cfg)
 				current_port = config_create_interface(port, cfg);
 				if (!current_port)
 					goto parse_error;
-				config_init_interface(current_port, cfg);
 			}
 			continue;
 		}
@@ -606,13 +607,11 @@ struct interface *config_create_interface(char *name, struct config *cfg)
 	}
 
 	strncpy(iface->name, name, MAX_IFNAME_SIZE);
-	STAILQ_INSERT_TAIL(&cfg->interfaces, iface, list);
-	return iface;
-}
-
-void config_init_interface(struct interface *iface, struct config *cfg)
-{
 	sk_get_ts_info(iface->name, &iface->ts_info);
+	STAILQ_INSERT_TAIL(&cfg->interfaces, iface, list);
+	cfg->n_interfaces++;
+
+	return iface;
 }
 
 struct config *config_create(void)
