@@ -41,6 +41,7 @@ struct interface {
 struct config {
 	/* configured interfaces */
 	STAILQ_HEAD(interfaces_head, interface) interfaces;
+	int n_interfaces;
 
 	/* hash of all non-legacy items */
 	struct hash *htab;
@@ -48,7 +49,6 @@ struct config {
 
 int config_read(char *name, struct config *cfg);
 struct interface *config_create_interface(char *name, struct config *cfg);
-void config_init_interface(struct interface *iface, struct config *cfg);
 void config_destroy(struct config *cfg);
 
 /* New, hash table based methods: */
