@@ -119,6 +119,8 @@ struct node {
 	struct clock *master;
 };
 
+static struct config *phc2sys_config;
+
 static int update_pmc(struct node *node, int subscribe);
 static int clock_handle_leap(struct node *node, struct clock *clock,
 			     int64_t offset, uint64_t ts);
@@ -222,7 +224,8 @@ static struct clock *clock_add(struct node *node, char *device)
 		}
 	}
 
-	c->servo = servo_create(node->servo_type, -ppb, max_ppb, 0);
+	c->servo = servo_create(phc2sys_config, node->servo_type,
+				-ppb, max_ppb, 0);
 	servo_sync_interval(c->servo, node->phc_interval);
 
 	if (clkid != CLOCK_REALTIME)
@@ -332,9 +335,14 @@ static void reconfigure(struct node *node)
 		}
 		last = c;
 	}
-	if (dst_cnt && !src) {
+	if (dst_cnt > 1 && !src) {
 		if (!rt || rt->dest_only) {
 			node->master = last;
+			/* Reset to original state in next reconfiguration. */
+			node->master->new_state = node->master->state;
+			node->master->state = PS_SLAVE;
+			if (rt)
+				rt->state = PS_SLAVE;
 			pr_info("no source, selecting %s as the default clock",
 				last->device);
 			return;
@@ -464,7 +472,7 @@ static void update_clock(struct node *node, struct clock *clock,
 	if (clock->sanity_check && clockcheck_sample(clock->sanity_check, ts))
 		servo_reset(clock->servo);
 
-	ppb = servo_sample(clock->servo, offset, ts, &state);
+	ppb = servo_sample(clock->servo, offset, ts, 1.0, &state);
 	clock->servo_state = state;
 
 	switch (state) {
@@ -772,13 +780,13 @@ static void send_subscription(struct node *node)
 	pmc_send_set_action(node->pmc, TLV_SUBSCRIBE_EVENTS_NP, &sen, sizeof(sen));
 }
 
-static int init_pmc(struct node *node, int domain_number)
+static int init_pmc(struct config *cfg, struct node *node, int domain_number)
 {
 	char uds_local[MAX_IFNAME_SIZE + 1];
 
 	snprintf(uds_local, sizeof(uds_local), "/var/run/phc2sys.%d",
 		 getpid());
-	node->pmc = pmc_create(TRANS_UDS, uds_local, 0, domain_number, 0, 1);
+	node->pmc = pmc_create(cfg, TRANS_UDS, uds_local, 0, domain_number, 0, 1);
 	if (!node->pmc) {
 		pr_err("failed to create pmc");
 		return -1;
@@ -1214,11 +1222,13 @@ int main(int argc, char *argv[])
 	char *progname;
 	char *src_name = NULL, *dst_name = NULL;
 	struct clock *src, *dst;
+	struct config *cfg;
 	int autocfg = 0, rt = 0;
 	int c, domain_number = 0, pps_fd = -1;
-	int r, wait_sync = 0;
+	int r = -1, wait_sync = 0;
 	int print_level = LOG_INFO, use_syslog = 1, verbose = 0;
-	double phc_rate;
+	int ntpshm_segment;
+	double phc_rate, tmp;
 	struct node node = {
 		.sanity_freq_limit = 200000000,
 		.servo_type = CLOCK_SERVO_PI,
@@ -1229,8 +1239,13 @@ int main(int argc, char *argv[])
 
 	handle_term_signals();
 
-	configured_pi_kp = KP;
-	configured_pi_ki = KI;
+	cfg = phc2sys_config = config_create();
+	if (!cfg) {
+		return -1;
+	}
+
+	config_set_double(cfg, "pi_proportional_const", KP);
+	config_set_double(cfg, "pi_integral_const", KI);
 
 	/* Process the command line arguments. */
 	progname = strrchr(argv[0], '/');
@@ -1252,7 +1267,7 @@ int main(int argc, char *argv[])
 			if (pps_fd < 0) {
 				fprintf(stderr,
 					"cannot open '%s': %m\n", optarg);
-				return -1;
+				goto end;
 			}
 			break;
 		case 'i':
@@ -1271,63 +1286,64 @@ int main(int argc, char *argv[])
 			} else {
 				fprintf(stderr,
 					"invalid servo name %s\n", optarg);
-				return -1;
+				goto end;
 			}
 			break;
 		case 'P':
-			if (get_arg_val_d(c, optarg, &configured_pi_kp,
-					  0.0, DBL_MAX))
-				return -1;
+			if (get_arg_val_d(c, optarg, &tmp, 0.0, DBL_MAX) ||
+			    config_set_double(cfg, "pi_proportional_const", tmp))
+				goto end;
 			break;
 		case 'I':
-			if (get_arg_val_d(c, optarg, &configured_pi_ki,
-					  0.0, DBL_MAX))
-				return -1;
+			if (get_arg_val_d(c, optarg, &tmp, 0.0, DBL_MAX) ||
+			    config_set_double(cfg, "pi_integral_const", tmp))
+				goto end;
 			break;
 		case 'S':
-			if (get_arg_val_d(c, optarg, &servo_step_threshold,
-					  0.0, DBL_MAX))
-				return -1;
+			if (get_arg_val_d(c, optarg, &tmp, 0.0, DBL_MAX) ||
+			    config_set_double(cfg, "step_threshold", tmp))
+				goto end;
 			break;
 		case 'F':
-			if (get_arg_val_d(c, optarg, &servo_first_step_threshold,
-					  0.0, DBL_MAX))
-				return -1;
+			if (get_arg_val_d(c, optarg, &tmp, 0.0, DBL_MAX) ||
+			    config_set_double(cfg, "first_step_threshold", tmp))
+				goto end;
 			break;
 		case 'R':
 			if (get_arg_val_d(c, optarg, &phc_rate, 1e-9, DBL_MAX))
-				return -1;
+				goto end;
 			node.phc_interval = 1.0 / phc_rate;
 			break;
 		case 'N':
 			if (get_arg_val_i(c, optarg, &node.phc_readings, 1, INT_MAX))
-				return -1;
+				goto end;
 			break;
 		case 'O':
 			if (get_arg_val_i(c, optarg, &node.sync_offset,
 					  INT_MIN, INT_MAX))
-				return -1;
+				goto end;
 			node.forced_sync_offset = -1;
 			break;
 		case 'L':
 			if (get_arg_val_i(c, optarg, &node.sanity_freq_limit, 0, INT_MAX))
-				return -1;
+				goto end;
 			break;
 		case 'M':
-			if (get_arg_val_i(c, optarg, &ntpshm_segment, INT_MIN, INT_MAX))
-				return -1;
+			if (get_arg_val_i(c, optarg, &ntpshm_segment, INT_MIN, INT_MAX) ||
+			    config_set_int(cfg, "ntpshm_segment", ntpshm_segment))
+				goto end;
 			break;
 		case 'u':
 			if (get_arg_val_ui(c, optarg, &node.stats_max_count,
 					  0, UINT_MAX))
-				return -1;
+				goto end;
 			break;
 		case 'w':
 			wait_sync = 1;
 			break;
 		case 'n':
 			if (get_arg_val_i(c, optarg, &domain_number, 0, 255))
-				return -1;
+				goto end;
 			break;
 		case 'x':
 			node.kernel_leap = 0;
@@ -1336,14 +1352,16 @@ int main(int argc, char *argv[])
 			if (strlen(optarg) > MAX_IFNAME_SIZE) {
 				fprintf(stderr, "path %s too long, max is %d\n",
 					optarg, MAX_IFNAME_SIZE);
-				return -1;
+				goto end;
+			}
+			if (config_set_string(cfg, "uds_address", optarg)) {
+				goto end;
 			}
-			strncpy(uds_path, optarg, MAX_IFNAME_SIZE);
 			break;
 		case 'l':
 			if (get_arg_val_i(c, optarg, &print_level,
 					  PRINT_LEVEL_MIN, PRINT_LEVEL_MAX))
-				return -1;
+				goto end;
 			break;
 		case 'm':
 			verbose = 1;
@@ -1353,9 +1371,11 @@ int main(int argc, char *argv[])
 			break;
 		case 'v':
 			version_show(stdout);
+			config_destroy(cfg);
 			return 0;
 		case 'h':
 			usage(progname);
+			config_destroy(cfg);
 			return 0;
 		default:
 			goto bad_usage;
@@ -1390,10 +1410,10 @@ int main(int argc, char *argv[])
 	print_set_level(print_level);
 
 	if (autocfg) {
-		if (init_pmc(&node, domain_number))
-			return -1;
+		if (init_pmc(cfg, &node, domain_number))
+			goto end;
 		if (auto_init_ports(&node, rt) < 0)
-			return -1;
+			goto end;
 		r = do_loop(&node, 1);
 		goto end;
 	}
@@ -1426,7 +1446,7 @@ int main(int argc, char *argv[])
 	r = -1;
 
 	if (wait_sync) {
-		if (init_pmc(&node, domain_number))
+		if (init_pmc(cfg, &node, domain_number))
 			goto end;
 
 		while (is_running()) {
@@ -1465,9 +1485,10 @@ int main(int argc, char *argv[])
 end:
 	if (node.pmc)
 		close_pmc(&node);
-
+	config_destroy(cfg);
 	return r;
 bad_usage:
 	usage(progname);
+	config_destroy(cfg);
 	return -1;
 }
