@@ -66,8 +66,8 @@ struct management_id idtab[] = {
 	{ "CURRENT_DATA_SET", TLV_CURRENT_DATA_SET, do_get_action },
 	{ "PARENT_DATA_SET", TLV_PARENT_DATA_SET, do_get_action },
 	{ "TIME_PROPERTIES_DATA_SET", TLV_TIME_PROPERTIES_DATA_SET, do_get_action },
-	{ "PRIORITY1", TLV_PRIORITY1, do_get_action },
-	{ "PRIORITY2", TLV_PRIORITY2, do_get_action },
+	{ "PRIORITY1", TLV_PRIORITY1, do_set_action },
+	{ "PRIORITY2", TLV_PRIORITY2, do_set_action },
 	{ "DOMAIN", TLV_DOMAIN, do_get_action },
 	{ "SLAVE_ONLY", TLV_SLAVE_ONLY, do_get_action },
 	{ "TIME", TLV_TIME, not_supported },
@@ -211,9 +211,9 @@ static void pmc_show(struct ptp_message *msg, FILE *fp)
 	}
 	tlv = (struct TLV *) msg->management.suffix;
 	if (tlv->type == TLV_MANAGEMENT) {
-		fprintf(fp, "MANAGMENT ");
+		fprintf(fp, "MANAGEMENT ");
 	} else if (tlv->type == TLV_MANAGEMENT_ERROR_STATUS) {
-		fprintf(fp, "MANAGMENT_ERROR_STATUS ");
+		fprintf(fp, "MANAGEMENT_ERROR_STATUS ");
 		goto out;
 	} else {
 		fprintf(fp, "unknown-tlv ");
@@ -493,6 +493,7 @@ static void do_get_action(int action, int index, char *str)
 static void do_set_action(int action, int index, char *str)
 {
 	struct grandmaster_settings_np gsn;
+	struct management_tlv_datum mtd;
 	struct port_ds_np pnp;
 	int cnt, code = idtab[index].code;
 	int leap_61, leap_59, utc_off_valid;
@@ -513,6 +514,24 @@ static void do_set_action(int action, int index, char *str)
 		return;
 	}
 	switch (code) {
+	case TLV_PRIORITY1:
+		cnt = sscanf(str,  " %*s %*s %hhu", &mtd.val);
+		if (cnt != 1) {
+			fprintf(stderr, "%s SET needs 1 value\n",
+				idtab[index].name);
+			break;
+		}
+		pmc_send_set_action(pmc, code, &mtd, sizeof(mtd));
+		break;
+	case TLV_PRIORITY2:
+		cnt = sscanf(str,  " %*s %*s %hhu", &mtd.val);
+		if (cnt != 1) {
+			fprintf(stderr, "%s SET needs 1 value\n",
+				idtab[index].name);
+			break;
+		}
+		pmc_send_set_action(pmc, code, &mtd, sizeof(mtd));
+		break;
 	case TLV_GRANDMASTER_SETTINGS_NP:
 		cnt = sscanf(str, " %*s %*s "
 			     "clockClass              %hhu "
@@ -719,11 +738,17 @@ int main(int argc, char *argv[])
 	enum transport_type transport_type = TRANS_UDP_IPV4;
 	UInteger8 boundary_hops = 1, domain_number = 0, transport_specific = 0;
 	struct ptp_message *msg;
+	struct config *cfg;
 #define N_FD 2
 	struct pollfd pollfd[N_FD];
 
 	handle_term_signals();
 
+	cfg = config_create();
+	if (!cfg) {
+		return -1;
+	}
+
 	/* Process the command line arguments. */
 	progname = strrchr(argv[0], '/');
 	progname = progname ? 1+progname : argv[0];
@@ -754,9 +779,13 @@ int main(int argc, char *argv[])
 			if (strlen(optarg) > MAX_IFNAME_SIZE) {
 				fprintf(stderr, "path %s too long, max is %d\n",
 					optarg, MAX_IFNAME_SIZE);
+				config_destroy(cfg);
+				return -1;
+			}
+			if (config_set_string(cfg, "uds_address", optarg)) {
+				config_destroy(cfg);
 				return -1;
 			}
-			strncpy(uds_path, optarg, MAX_IFNAME_SIZE);
 			break;
 		case 't':
 			if (1 == sscanf(optarg, "%x", &c))
@@ -764,18 +793,19 @@ int main(int argc, char *argv[])
 			break;
 		case 'v':
 			version_show(stdout);
+			config_destroy(cfg);
 			return 0;
 		case 'z':
 			zero_datalen = 1;
 			break;
 		case 'h':
 			usage(progname);
+			config_destroy(cfg);
 			return 0;
 		case '?':
-			usage(progname);
-			return -1;
 		default:
 			usage(progname);
+			config_destroy(cfg);
 			return -1;
 		}
 	}
@@ -797,10 +827,11 @@ int main(int argc, char *argv[])
 	print_set_syslog(1);
 	print_set_verbose(1);
 
-	pmc = pmc_create(transport_type, iface_name, boundary_hops,
+	pmc = pmc_create(cfg, transport_type, iface_name, boundary_hops,
 			 domain_number, transport_specific, zero_datalen);
 	if (!pmc) {
 		fprintf(stderr, "failed to create pmc\n");
+		config_destroy(cfg);
 		return -1;
 	}
 
@@ -876,5 +907,6 @@ int main(int argc, char *argv[])
 
 	pmc_destroy(pmc);
 	msg_cleanup();
+	config_destroy(cfg);
 	return ret;
 }
