@@ -117,6 +117,7 @@ struct port {
 	int                 path_trace_enabled;
 	int                 rx_timestamp_offset;
 	int                 tx_timestamp_offset;
+	int                 link_status;
 	struct fault_interval flt_interval_pertype[FT_CNT];
 	enum fault_type     last_fault_type;
 	unsigned int        versionNumber; /*UInteger4*/
@@ -145,7 +146,7 @@ static int announce_compare(struct ptp_message *m1, struct ptp_message *m2)
 	return memcmp(&a->grandmasterPriority1, &b->grandmasterPriority1, len);
 }
 
-static void announce_to_dataset(struct ptp_message *m, struct clock *c,
+static void announce_to_dataset(struct ptp_message *m, struct port *p,
 				struct dataset *out)
 {
 	struct announce_msg *a = &m->announce;
@@ -155,7 +156,7 @@ static void announce_to_dataset(struct ptp_message *m, struct clock *c,
 	out->priority2    = a->grandmasterPriority2;
 	out->stepsRemoved = a->stepsRemoved;
 	out->sender       = m->header.sourcePortIdentity;
-	out->receiver     = clock_parent_identity(c);
+	out->receiver     = p->portIdentity;
 }
 
 static int msg_current(struct ptp_message *m, struct timespec now)
@@ -2032,7 +2033,7 @@ struct foreign_clock *port_compute_best(struct port *p)
 		if (!tmp)
 			continue;
 
-		announce_to_dataset(tmp, p->clock, &fc->dataset);
+		announce_to_dataset(tmp, p, &fc->dataset);
 
 		fc_prune(fc);
 
@@ -2357,6 +2358,17 @@ int port_number(struct port *p)
 	return portnum(p);
 }
 
+int port_link_status_get(struct port *p)
+{
+	return p->link_status;
+}
+
+void port_link_status_set(struct port *p, int up)
+{
+	p->link_status = up ? 1 : 0;
+	pr_notice("port %hu: link %s", portnum(p), up ? "up" : "down");
+}
+
 int port_manage(struct port *p, struct port *ingress, struct ptp_message *msg)
 {
 	struct management_tlv *mgt;
@@ -2573,6 +2585,7 @@ struct port *port_open(int phc_index,
 	p->path_trace_enabled = config_get_int(cfg, p->name, "path_trace_enabled");
 	p->rx_timestamp_offset = config_get_int(cfg, p->name, "ingressLatency");
 	p->tx_timestamp_offset = config_get_int(cfg, p->name, "egressLatency");
+	p->link_status = 1;
 	p->clock = clock;
 	p->trp = transport_create(cfg, transport);
 	if (!p->trp)
