@@ -39,6 +39,12 @@ struct sk_ts_info {
 	unsigned int rx_filters;
 };
 
+/**
+ * Obtains a socket suitable for use with sk_interface_index().
+ * @return  An open socket on success, -1 otherwise.
+ */
+int sk_interface_fd(void);
+
 /**
  * Obtain the numerical index from a network interface by name.
  * @param fd      An open socket.
